/*
 *
 * happytimer - Alarm clock for the GNOME
 *
 *  @Copyright (C) 2007-2011 JinCheol Kim (shriekout@gmail.com).
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 */

#include <gtk/gtk.h>
#include <glib/gi18n.h>
#include <config.h>
#include "interface.h"
#include "callback.h"
#include "happytimer.h"
#include "timecheck.h"

GtkStatusIcon *create_statusicon(void)
{
	GtkStatusIcon *icon;

	icon= gtk_status_icon_new_from_file(LOGO_FILE);
	gtk_status_icon_set_tooltip_text(icon, _("Happytimer"));
	g_signal_connect(G_OBJECT(icon), "button-press-event",
					G_CALLBACK(on_icon_button_press_event), NULL);

	return icon;
}

/* status 아이콘에서 오른쪽 버튼 클릭시 나타날 메뉴 */
GtkWidget *create_menu(void)
{
	GtkWidget *menu;
	GtkWidget *timer_menu;
	GdkPixbuf *pixbuf;
	GtkWidget *image;
	GtkWidget *timer_preferences;
	GtkWidget *stop_sound;
	GtkWidget *timer_about;
	GtkWidget *separator_menu;
	GtkWidget *quit_menu;
	GtkAccelGroup *accel_group;
	gint x, y;

	accel_group = gtk_accel_group_new();

	menu = gtk_menu_new();

	timer_menu = gtk_image_menu_item_new_with_label(_("timer"));
	gtk_icon_size_lookup(GTK_ICON_SIZE_BUTTON, &x, &y);
	pixbuf = gdk_pixbuf_new_from_file_at_size(LOGO_FILE, x, y, NULL);

	image = gtk_image_new_from_pixbuf(pixbuf);
	gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(timer_menu), image);
	gdk_pixbuf_unref(pixbuf);

	gtk_widget_show(image);
	gtk_widget_show(timer_menu);
	gtk_container_add(GTK_CONTAINER(menu), timer_menu);
	g_signal_connect(G_OBJECT(timer_menu), "activate",
			G_CALLBACK(display_timer), NULL);

	timer_preferences = gtk_image_menu_item_new_from_stock("gtk-preferences", accel_group);
	gtk_container_add(GTK_CONTAINER(menu), timer_preferences);
	gtk_widget_show(timer_preferences);
	g_signal_connect(G_OBJECT(timer_preferences), "activate",
			G_CALLBACK(display_preferences), (gpointer)FALSE);

	stop_sound = gtk_image_menu_item_new_with_label(_("Stop sound!"));
	gtk_container_add(GTK_CONTAINER(menu), stop_sound);
	gtk_widget_show(stop_sound);
	g_signal_connect(G_OBJECT(stop_sound), "activate",
			G_CALLBACK(stop_sound_cb), NULL);

	timer_about = gtk_image_menu_item_new_from_stock("gtk-about", accel_group);
	gtk_widget_show(timer_about);
	gtk_container_add(GTK_CONTAINER(menu), timer_about);
	g_signal_connect(G_OBJECT(timer_about), "activate",
			G_CALLBACK(display_about_dialog), NULL);

	separator_menu = gtk_separator_menu_item_new();
	gtk_widget_show(separator_menu);
	gtk_container_add(GTK_CONTAINER(menu), separator_menu);
	gtk_widget_set_sensitive(separator_menu, TRUE);

	quit_menu = gtk_image_menu_item_new_from_stock("gtk-quit", accel_group);
	gtk_widget_show(quit_menu);
	gtk_container_add(GTK_CONTAINER(menu), quit_menu);
	g_signal_connect(G_OBJECT(quit_menu), "activate",
			G_CALLBACK(destroy), NULL);

	gtk_menu_set_accel_group(GTK_MENU(menu), accel_group);

	return menu;
}

static void load_preset(GtkWidget *widget)
{
	struct presetList *cur;
	char *time, buf[80];

	cur = presetList_head;

	while (cur) {
		time = return_format_date((time_t)cur->timer);
		sprintf(buf, "%s (%s)", cur->name, time);
		free(time);

		gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(widget), buf);

		cur = cur->next;
	}

	gtk_combo_box_set_active(GTK_COMBO_BOX(widget), 0);
}

GtkWidget *create_preferencesdialog(void)
{
	GtkBuilder *builder;
	GtkWidget *dialog;
	GError *error = NULL;
	GtkWidget *widget;
	GtkWidget *stop_button;
	GtkAdjustment *adj;
	GdkPixbuf *pixbuf;
	GtkFileFilter *audio_filter, *all_filter;
	char *audio_mime_types[] = {
		"audio/3gpp",
		"audio/x-mp3",
		"audio/x-mpeg",
		"audio/mpeg",
		"audio/x-wav",
		"audio/x-mpegurl",
		"audio/mpegurl",
		"audio/x-scpls",
		"audio/x-m4a",
		"audio/x-ms-asf",
		"audio/x-ms-asx",
		"audio/x-ms-wax",
		"audio/x-real-audio",
		"audio/x-pn-realaudio",
		"application/x-flac",
		"audio/x-flac",
		"audio/vnd.rn-realaudio",
		"audio/x-pn-aiff",
		"audio/x-pn-au",
		"audio/x-pn-wav",
		"audio/x-pn-windows-acm",
		"audio/ogg"
	};
	int count;

	builder = gtk_builder_new();
	if (!gtk_builder_add_from_file(builder, UI_DIR"/preferences_dialog.glade", &error)) {
		g_warning(_("Could't load builder file: %s"), error->message);
		g_error_free(error);
	}

	gtk_builder_connect_signals(builder, NULL);
	dialog = GTK_WIDGET(gtk_builder_get_object(builder, "preferences_dialog"));

	gtk_dialog_add_button(GTK_DIALOG(dialog),
						GTK_STOCK_CLOSE,
						GTK_RESPONSE_CLOSE);

	audio_filter = gtk_file_filter_new();
	gtk_file_filter_set_name(audio_filter, _("Audio Files"));
	for (count = 0; count < G_N_ELEMENTS(audio_mime_types); count++)
		gtk_file_filter_add_mime_type(audio_filter, audio_mime_types[count]);

	all_filter = gtk_file_filter_new();
	gtk_file_filter_set_name(all_filter, _("All files"));
	gtk_file_filter_add_pattern(all_filter, "*");

	widget = GTK_WIDGET(gtk_builder_get_object(builder, "rtsound_checkbutton"));
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widget), prefer.rtsound);

	widget = GTK_WIDGET(gtk_builder_get_object(builder, "rtsound_filechooserbutton"));
	gtk_file_chooser_set_filename(GTK_FILE_CHOOSER(widget), prefer.sound_file[RT_SND]);
	gtk_file_chooser_add_filter(GTK_FILE_CHOOSER(widget), audio_filter);
	gtk_file_chooser_add_filter(GTK_FILE_CHOOSER(widget), all_filter);

	stop_button = gtk_button_new_from_stock(GTK_STOCK_MEDIA_STOP);
	gtk_widget_show(stop_button);
	gtk_file_chooser_set_extra_widget(GTK_FILE_CHOOSER(widget), stop_button);
	g_signal_connect(G_OBJECT(stop_button), "clicked",
			G_CALLBACK(on_preview_stop_button_clicked), NULL);

	rtplay_togglebutton = GTK_WIDGET(gtk_builder_get_object(builder, "rtplay_togglebutton"));

	widget = GTK_WIDGET(gtk_builder_get_object(builder, "tsound_checkbutton"));
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widget), prefer.tsound);

	widget = GTK_WIDGET(gtk_builder_get_object(builder, "tsound_filechooserbutton"));
	gtk_file_chooser_set_filename(GTK_FILE_CHOOSER(widget), prefer.sound_file[ARM_SND]);
	gtk_file_chooser_add_filter(GTK_FILE_CHOOSER(widget), audio_filter);
	gtk_file_chooser_add_filter(GTK_FILE_CHOOSER(widget), all_filter);

	stop_button = gtk_button_new_from_stock(GTK_STOCK_MEDIA_STOP);
	gtk_widget_show(stop_button);
	gtk_file_chooser_set_extra_widget(GTK_FILE_CHOOSER(widget), stop_button);
	g_signal_connect(G_OBJECT(stop_button), "clicked",
			G_CALLBACK(on_preview_stop_button_clicked), NULL);

	tplay_togglebutton = GTK_WIDGET(gtk_builder_get_object(builder, "tplay_togglebutton"));

	adj = GTK_ADJUSTMENT(gtk_builder_get_object(builder, "notify_adjustment"));
	gtk_adjustment_set_value(adj, (prefer.notify_before/60.0));
	
	preview_image = GTK_WIDGET(gtk_builder_get_object(builder, "background_image"));
	pixbuf = gdk_pixbuf_new_from_file_at_size(prefer.timage_file, 128, 128, NULL);
	gtk_image_set_from_pixbuf(GTK_IMAGE(preview_image), pixbuf);
	gdk_pixbuf_unref(pixbuf);

	preset_config_comboboxtext = GTK_WIDGET(gtk_builder_get_object(builder, "preset_config_comboboxtext"));
	load_preset(preset_config_comboboxtext);

	g_object_unref(builder);

	return dialog;
}

void load_timer_window(void)
{
	GtkBuilder *builder;
	GError *error = NULL;
	GdkColor color;

	builder = gtk_builder_new();
	if (!gtk_builder_add_from_file(builder, UI_DIR"/timer.glade", &error)) {
	g_warning(_("Could't load builder file: %s"), error->message);
		g_error_free(error);
	}

	gtk_builder_connect_signals(builder, NULL);

	timer_window = GTK_WIDGET(gtk_builder_get_object(builder, "timer_window"));

	gtk_window_set_position(GTK_WINDOW(timer_window), GTK_WIN_POS_MOUSE);
	gtk_window_set_decorated(GTK_WINDOW(timer_window), FALSE);
	gtk_window_stick(GTK_WINDOW(timer_window));
	gtk_window_set_keep_above(GTK_WINDOW(timer_window), TRUE);
	gtk_window_set_skip_taskbar_hint(GTK_WINDOW(timer_window), TRUE);

	hrs_spinbutton = GTK_WIDGET(gtk_builder_get_object(builder, "hrs_spinbutton"));
	mins_spinbutton = GTK_WIDGET(gtk_builder_get_object(builder, "mins_spinbutton"));
	secs_spinbutton = GTK_WIDGET(gtk_builder_get_object(builder, "secs_spinbutton"));
	timerleft_label = GTK_WIDGET(gtk_builder_get_object(builder, "timerleft_label"));
	start_button = GTK_WIDGET(gtk_builder_get_object(builder, "start_button"));
	start_button_image = GTK_WIDGET(gtk_builder_get_object(builder, "start_button_image"));
	minus_30_button = GTK_WIDGET(gtk_builder_get_object(builder, "minus_30_button"));
	plus_30_button = GTK_WIDGET(gtk_builder_get_object(builder, "plus_30_button"));
	pause_button = GTK_WIDGET(gtk_builder_get_object(builder, "pause_button"));
	preset_comboboxtext = GTK_WIDGET(gtk_builder_get_object(builder, "preset_comboboxtext"));

	load_preset(preset_comboboxtext);

	gdk_color_parse("black", &color);
	gtk_widget_modify_bg(timer_window, GTK_STATE_NORMAL, &color);

	gtk_widget_set_sensitive(pause_button, FALSE);

	g_object_unref(builder);
}

static void on_alpha_screen_changed(GtkWidget *widget)
{
	GdkScreen *screen;
	GdkVisual *visual;

	screen = gtk_widget_get_screen(widget);
	visual = gdk_screen_get_rgba_visual(screen);

	gtk_widget_set_visual(widget, visual);
}

static gboolean on_alpha_widget_draw(GtkWidget *widget, cairo_t *cr)
{
	if (gdk_screen_get_rgba_visual(gtk_widget_get_screen(widget)) &&
				gtk_widget_is_composited(widget))
		cairo_set_source_rgba(cr, 1.0, 1.0, 1.0, 0.0);
	else
		cairo_set_source_rgb(cr, 1.0, 1.0, 1.0);

	cairo_set_operator(cr, CAIRO_OPERATOR_SOURCE);
	cairo_paint(cr);

	return FALSE;
}

/* 타이머 종료시 나타날 창 */
GtkWidget *create_alarm_window(void)
{
	GtkWidget *window;
	GtkWidget *event;
	GtkWidget *image;

	window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	gtk_window_set_resizable(GTK_WINDOW(window), FALSE);
	gtk_window_set_position(GTK_WINDOW(window), GTK_WIN_POS_CENTER_ALWAYS);
	gtk_window_set_decorated(GTK_WINDOW(window), FALSE);
	gtk_window_stick(GTK_WINDOW(window));
	gtk_window_set_keep_above(GTK_WINDOW(window), TRUE);
	gtk_window_set_skip_taskbar_hint(GTK_WINDOW(window), FALSE);

	event = gtk_event_box_new();
	image = gtk_image_new_from_file(prefer.timage_file);

	gtk_container_add(GTK_CONTAINER(window), event);
	g_signal_connect(event, "button_press_event",
					G_CALLBACK(alarm_window_destroy), NULL);

	gtk_container_add(GTK_CONTAINER(event), image);

	gtk_widget_set_app_paintable(window, TRUE);
	gtk_widget_set_app_paintable(event, TRUE);

	g_signal_connect(window, "draw",
					G_CALLBACK(on_alpha_widget_draw), NULL);
	g_signal_connect(event, "draw",
					G_CALLBACK(on_alpha_widget_draw), NULL);

	on_alpha_screen_changed(window);
	on_alpha_screen_changed(event);

	gtk_widget_show_all(window);

	return window;
}
